//
// C Interface: stationir
//
// Description:
//
//
// Author: Leonhard Klein <leoklein@gmx.net>, (C) 2004
#ifndef __STATIONIR_H__
#define __STATIONIR_H__

#include <avr/io.h>
#include "basic.h"


#define IRMLENGTH 20                     /*!< maximum pulse length in bytes */
#define IRMSLENGTH 30                    /*!< maximum string length in bytes */
#define IRDATABITTIME 12                 /*!< bit time in 36kHz cycles for data transmiting */
#define IRDATABYTETIME 128               /*!< time to wait while a byte is transmitted */
#define IRKEYS 4                         /*!< number of recognized keys <=5 */

#ifdef TEST
  #define ISIRON (1)
  #define WAITIR
#else
  #define ISIRON ((PIND & _BV(PD4))==0)    /*!< 1: ir-on, 0: ir-off */
  #define WAITIR {sei(); while (sendready == 0) _NOP; cli(); } while (0)
#endif

typedef u08 irdatatype[IRMLENGTH];

u08 irerror;                             //!< error code of last ir pulse conversation
u08 irbittime;                           //!< length of one bit in 36kHz cycles

irdatatype cmp_target[IRKEYS];           //!< datas read from eeprom for ircont (ir_load())

extern volatile irdatatype irrbuf;       //!< received single ir-pulse ("<length>blablabla")
extern volatile u08 irrstring[IRMSLENGTH];//!< received string. ("<length>blablabla")
extern volatile u08 pulsevalid;
extern volatile u08 stringvalid;
extern volatile u08 sendready;
extern void ir_send(u08 *irbuf_);        //!< sends one pulse of ir ("<length>blablabla")
extern void ir_string_send(u08 *se);     //!< sends a string of ir-datas ( "blablabla\0")

extern u08 ir_comp(irdatatype cmp_target, irdatatype read_ir);
#ifdef STATION
extern void ir_and(irdatatype res, irdatatype and2);
#endif

/* don't use in do_*' s */
extern volatile u08 *senddata;           //!< pointer to the data to send just for the INT-function
extern u08 ir_to_val(irdatatype data);
extern u08 *val_to_ir(u08 dat);
extern void do_ir();                     //!< handles the lowlevel tranmission and receiving
extern void start_ir_r();                //!< starts the receiving of a pulse
extern void do_ir_string();              //!< handles string sending

#endif
