//
// C main: robot
//
// Description: Program of the leobot
//
//
// Author: Leonhard Klein <leoklein@gmx.net>, (C) 2004
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/eeprom.h>

#include "robotsound.h"
#include "robotu.h"
#include "robotdo.h"
#include "basic.h"
#include "ir.h"


SIGNAL(SIG_ADC) {
  static u08 irfrcount;
  static s16 irposstart;
  switch (admodus) {
  case irpos:                                      /* startup free running 4,8kHz */
    irfrcount = 0;
    admodus = irfr2;
    irposstart = ADC;
    return;
  case irfr2:                                      /* normal free running */
    irfrcount++;
    if (irfrcount < 20) {
      PORTB ^= _BV(PB4);
      return;
    } else {
      irpos_value = ADC - irposstart;
      PORTB &= ~_BV(PB4);
    }
    break;
  case irstation:
    irstation_value = ADC;
    break;
  case light:
    light_value = ADC >> 2;
    break;
  case vcc:
    vcc_value = 1000 - ADC;
    break;
  case vcco:
    vcco_value = ADC;
    break;
  }
  ADCSRA = 0;                                                           /* disable AD-Converter */
  admodus = none;
}

SIGNAL(SIG_OVERFLOW0) {                                                  /* Timer0 Overflow for threating */
  static u08 odp;
  struct measuretype mmodus;
  cli();
  if ((modus == ir_r)|(modus == ir_s)) {                                /* common check for common timing */
    TCNT0 = 255-111;                                                    /* 13.888 = 36000Hz */
    do_ir();
    sei();
    return;
  }
  if (modus == radar) {
    if (radar_value++ == 0)
      PORTD |= _BV(PD5);                                                /* set INIT */
    if ((radar_value > 1000) || (PINC & _BV(PC2))) {                    /* got ECHO or timeout */
      TCCR0 = _BV(CS01);                                                /* 2kHz - normal operation */
      modus = oldmodus;
      PORTD &= ~_BV(PD5);
      PORTD |= _BV(PD6);                                                /* switch radar off */
    }
    sei();
    return;
  }
  /* normal operation */
  time ++;
  decode_position();
  mmodus = measuremodus[modus];
  if                        (time % _BV(5) ==  0)   playi();          /* 1,935kHz / 32  = 60Hz  */
  if                        (time % _BV(7) ==  1)   drive();          /* 1,935kHz / 128 = 12Hz */
  if ((mmodus.light)     & ((time % _BV(8) ==  2))) start_ad(light);  /* 1,935kHz / 256 = 6Hz */
  if ((mmodus.vcc)       & ((time % _BV(8) ==  4))) start_ad(vcc);
  if ((mmodus.vcco)      & ((time % _BV(8) ==  6))) start_ad(vcco);
  if ((mmodus.irstation) & ((time % _BV(6) ==  8))) start_ad(irstation);
  if ((mmodus.radar)     & ((time % _BV(10)==  9))) {
    start_radar();
    return;
  }
  if ((mmodus.irpos)     & ((time % _BV(7) == 10))) start_ad(irpos);  /* 10: leave maximum time to rest for irstation */
  if ((mmodus.irr)       & (ISIRON)) {
    start_ir_r();
    return;
  }
  switch (modus) {
  case irstring_s:
    do_ir_string();
    break;
  case boredom:
    do_boredom();
    break;
  case sleep:
    do_sleep();
    break;
  case connected:
    do_connected();
    break;
  case searching:
    do_searching();
    break;
  case flee:
    do_flee();
    break;
  case empty:
    do_empty();
    break;
#ifdef SCRIPT
  case script:
    do_script();
    break;
#endif
#ifdef IRCONT
  case ircont:
    do_ircont();
    break;
#endif
  default: SETMODUS (boredom);
  }
  if (vcc_value < BATLOW) needpower = 1;
  if (vcc_value < BATEMPTY) {            /* rescuemodus */
    RESETP;
    modus = empty;
  }
  olddippos = DIPPOS;
  TCNT0 = 0;
  sei();
}

int main(void) {
  u16 i;
#ifndef TEST
  for (i=0; i < 60000;i++) _NOP;
#endif
  admodus = none;
  vcc_value = BATFULL;
  light_value = 255;
  start_sound(start_up, 1);
  switch (olddippos = DIPPOS) {
  case 0: modus = boredom; break;
  case 1: modus = connected; break;
  case 2: modus = ircont; break;
  case 3: modus = script; break;
  }
  /*  init_pins */
  DDRB  = _BV(DDB0)|_BV(DDB1)|_BV(DDB2)|_BV(DDB3)|_BV(DDB4);
  DDRD  = _BV(DDD5)|_BV(DDD6)|_BV(DDD7);
  PORTD = _BV(PD0)|_BV(PD1)|_BV(PD2)|_BV(PD3);          /* enable pull-up's for angle sensors */
  /*  init_int */
  TIMSK = TIMSK | _BV(TOIE0);
  TCNT0 = 0;
  TCCR0 = _BV(CS01);                                    /* Clock / 8 intervall is 4m/256*8~=2k */
  /*  init_pwm */
  TCCR1A= _BV(COM1A1)|_BV(COM1B1)|_BV(WGM10);           /* PWM 8bit*/
  TCCR1B= _BV(CS11)|_BV(CS10);                          /*  /64 */
  TCNT1 = 0;
  OCR1A = 0;
  OCR1B = 0;                                            /* reset timers */

  sei();                                   /* enable interrupts */
  for (;;)
    ;                                      /* loop forever */
}
