//
// C Implementation: robotsound
//
// Description: sound routines for robot
//
//
// Author: Leonhard Klein <leoklein@gmx.net>, (C) 2004
//
#include "robotsound.h"
/* (time, tone)* */
u08 __ATTR_PROGMEM__ beeps[] = {5,40, 10,0, 5,40, 10,0, 5,40, 10,0, 5,40, 10,0, 5,40, 10,0,
                                5,40, 10,0, 5,40, 10,0, 5,40, 10,0, 5,40, 10,0, 5,40, 40,0, 0,0}; /* ten beeps */
u08 __ATTR_PROGMEM__ start_up[] = {5,42, 3,54, 10,0, 0,0};
u08 __ATTR_PROGMEM__ go_sleep[] = {3,70, 3,60, 3,50, 3,40, 3,30, 3,20, 0,0};
u08 __ATTR_PROGMEM__ wake_up[] = {3,20, 3,30, 3,40, 3,50, 3,60, 3,70, 0,0};
u08 __ATTR_PROGMEM__ go_hungry[] = {20,36, 20,48, 20,36, 20,48, 0,0};
u08 __ATTR_PROGMEM__ go_empty[] = {10,49, 255,0, 10,49, 255,0, 10,49, 0,0};
u08 __ATTR_PROGMEM__ snore[] = {50,1, 40,0, 0,0};
u08 __ATTR_PROGMEM__ get_connected[] = {1,20, 50,85, 50,86, 10,70, 0,0};
u16 __ATTR_PROGMEM__ tones[] = { 0,
                                 33,   35,   37,   39,   41,   44,   46,   49,   52,   55,   58,   62,
                                 65,   69,   73,   78,   82,   87,   92,   98,   104,  110,  117,  123,
                                 131,  139,  147,  156,  164,  175,  185,  196,  208,  220,  233,  247,
                                 262,  277,  294,  311,  330,  349,  370,  392,  415,  440,  466,  494,
                                 523,  554,  587,  622,  659,  698,  740,  784,  831,  880,  932,  988,
                                 1047, 1109, 1175, 1245, 1319, 1397, 1480, 1568, 1661, 1760, 1865, 1976,
                                 2093, 2217, 2349, 2489, 2637, 2794, 2960, 3136, 3322, 3520, 3729, 3951}; /* 84 tones */

u08 *act_sound;
s16 s_ticks;                                                   /* sound ticks to next tone */

void play(u16 Hz) {
  u08 tccr2tmp;
  if (Hz < 17) {
    tccr2tmp = 0;
  } else if (Hz < 61) {
    tccr2tmp = _BV(CS22)|_BV(CS21)|_BV(CS20);
    OCR2 = 4000000/1024 / Hz;
  } else if (Hz < 124) {
    tccr2tmp = _BV(CS22)|_BV(CS21);
    OCR2 = 4000000/256 / Hz;
  } else if (Hz < 247) {
    tccr2tmp = _BV(CS22)|_BV(CS20);
    OCR2 = 4000000/128 / Hz;
  } else if (Hz < 493) {
    tccr2tmp = _BV(CS22);
    OCR2 = 4000000/64 / Hz;
  } else if (Hz < 1968) {
    tccr2tmp = _BV(CS21)|_BV(CS20);
    OCR2 = 4000000/32 / Hz;
  } else {
    tccr2tmp = _BV(CS21);
    OCR2 = 4000000/8 / Hz;
  }
  if (TCNT2 > OCR2) TCNT2 = 0;
  TCCR2 = tccr2tmp | _BV(WGM21) | _BV(COM20);
}

void playt(u08 tone) {
  static u08 ltone;
  if (tone < 84)
    ltone = tone;
  else {
    switch (tone) {
    case 85:
      if (ltone++ > 84) ltone = 1;
      break;
    case 86:
      if (--ltone < 2) ltone = 84;
      break;
    case 87:
      break;
    }
  }
  play(pgm_read_word_near(&tones[ltone]));
}

int playi() {                                                  /* called from Timer0 */
  static u08 ttplay;
  if (!soundplaying) return;
  if (s_ticks>0) {
    s_ticks--;
  } else {
    s_ticks = pgm_read_byte_near(&act_sound[0]);
    ttplay = pgm_read_byte_near(&act_sound[1]);
    act_sound +=2;
    if (!s_ticks) {
      TIMSK &= ~_BV(TOIE2);
      soundplaying = 0;
    }
  }
  playt(ttplay);
}

int start_sound(u08 *soundtostart, u08 override) {
  if (soundplaying && !override) return;
  TIMSK |= _BV(TOIE2);                                         /* enable TCNT2 overflow */
  act_sound = soundtostart;
  s_ticks = 0;
  soundplaying = 1;
}
