//
// C Interface: robotu
//
// Description: helper routines for robot.c
//
//
// Author: Leonhard Klein <leoklein@gmx.net>, (C) 2004
#ifndef __ROBOT_H__
#define __ROBOT_H__

#include "robotsound.h"
#include "basic.h"
#include "ir.h"

#define POS360 1187
/* 1187 ticks = 56,5cm; 1 tick = 0.476mm; about 460ticks per round */
#define MINSPEED 10
#define BATLOW   173 /*!< vcc_value at low battery, about 4V */
#define BATEMPTY 120 /*!< reeeally empty: Modus empty 3,5V*/
#define BATFULL 365  /*!< after loading and disconnect: 5,9V for a full battery */
#define IRMLENGTH 20 /*!< the maximum length for a IR-string */
/* some battery values:
4.5: 244
5.0: 292
5.5: 338
*/

#define INITSKRIPT 102    /*!< the first eeprom adress for the skript */
#define DIVSPEED 85       /*!< the divisor vor the speed.*/


u16 time, modustime;

u08 light_value;                                        /* to store the results of the AD */
u16 vcc_value, irpos_value, vcco_value;                 /* to store ... in 1/100V */
s16 radar_value, irstation_value;                       /* holds time from last radar-echo */


#define DIPPOS ((PINC & (_BV(PC3)|_BV(PC4)))>>3)
#define SETMODUS(nmodus) do { modus=nmodus; modustime = 0; l_to=positionl; r_to=positionr; } while (0)
#define RESETP do { l_to=positionl; r_to=positionr; speed = 0; } while (0)


#define ISIRON ((PIND & _BV(PD4))==0)    /*!< 1: ir-on, 0: ir-off */


s16 heading, positionl, positionr;                      /*< from position sensor; heading counterclockwise */
s08 speed, direction;                                   /*< for driving, use directly */
s16 l_to, r_to;                                         /*< for driving, use with drive_to_pos */
u16 d_pos;                                              /*< the difference in ticks to the target position */
u08 olddippos;                                          /*< the dip-switch position of the last call of the do'functions */
u08 needpower;                                          /*< is set for loading when reaching station next time */

#ifdef SINUS
extern s08 sine(u08 in);
#define COSINE(x) sine(x+90%255)
#endif
extern void decode_position();                                  /* decode angle sensors */
extern void drive_to_pos(s16 l2,s16 r2);
extern void drive();                                            /* set H-Bridge according to speed and dire*/

extern void start_ad(enum admodtype adm);
extern void start_radar();
extern u08 atoi(u08 *str);

extern void ir_int_send(u16 val);
#endif
