//
// C Implementation: ir for robot and station
//
// Description:
//
//
// Author: Leonhard Klein <leoklein@gmx.net>, (C) 2004
#include "ir.h"
#include <avr/interrupt.h>
#include <avr/eeprom.h>

#define IRON   (PORTB |= _BV(PB4))
#define IROFF  (PORTB &= ~_BV(PB4))
#define IRTOGGLE (PORTB ^= _BV(PB4))

u16 irtime;
s16 irbit;

volatile irdatatype irrbuf;                                              //< here the received irpulse is stored
volatile u08 irrstring[IRMSLENGTH];                                      //< here the received irdatas are stored

volatile u08 *irsbuf;                                                    //! pointer to the pulse to send
volatile u08 *senddata;                                                  //! pointer to the data to send
volatile u08 pulsevalid;
volatile u08 stringvalid;
volatile u08 sendready;

irdatatype cmp_target[IRKEYS];


u08 ir_to_val(irdatatype data) {
  u08 i, ibitcount, r, rb;
#define DATA(x) (data[x>>3]&(1<<(7-(x%8))))
  r = 0;
  rb = 0;
  ibitcount = 0;
  irerror=0;
  i = 12; while (DATA(i)!=0) i++;                                                   // find '0'
  for (; i < (data[0]*8)+1; i++) {
    if (DATA(i)) ibitcount++;                                                      // count '1'
    else {
      if (ibitcount >= 2) {
        if ((rb == 8)) {
          for (i=0;i<=7;i++) if (r & (1 << i)) rb++;
          irerror = ((rb%2)^(ibitcount <= 4));
          return r;
        }
        r |= (1<<rb++)*(ibitcount > 4);
      }
      ibitcount = 0;
    }
  }
  irerror = 2;
  return r;
}

u08 *val_to_ir(u08 dat) {
  u08 i;
  static irdatatype irdata;
  irdata[0] = 12;
  irdata[1] = 0xFF;
  irdata[2] = 0xF0;
  irdata[11] = 0x00;
  for (i=0; i<8; i++)
    if ((dat & _BV(i)) == 0)
      irdata[i + 3] = 0xE0; //0b1110 0000
    else {
      irdata[i + 3] = 0xFC; //0b1111 1100
      irdata[11] ^= 0xFF;
    }
  irdata[11]=(irdata[11]==0xFF)?0xE0:0xFC; //gerade: 0, ungerade: 1
  return irdata;
}

void do_ir() {
#define IRSBUF(x) (irsbuf[x>>3]&(1<<(7-(x%8))))
  static u08 irbiton;
  u08 i;
  if (modus == ir_s) {
    if (irtime++ >= irbittime) {
      irtime = 0;
      irbiton = (IRSBUF(irbit) != 0);
      irbit++;
    }
    if (irbiton)
      IRTOGGLE;
    else
      IROFF;
    if (irbit == irsbuf[0]*8) {
      TCCR0 = _BV(CS01);
      modus = oldmodus;
    }
  } else                                                  /* receive */
    if (irtime++ >= irbittime) {
      irtime=0;
      if (ISIRON) {
        irbiton = 0;
        irrbuf[irbit>>3] |= (1<<(7-(irbit % 8)));
      } else {
        if (irbiton++ == 20) {
          irrbuf[0] = (irbit>>3)-1;
          irbit = (IRMLENGTH<<3)+3;
        }
      }
      irbit++;
      if (irbit > (IRMLENGTH<<3)) {
        pulsevalid = 1;
        i = ir_to_val(irrbuf);
        if ((!irerror) & (irrbuf[0] < 15)) {
          if (irrstring[0] < IRMSLENGTH) irrstring[0]++;  /* irrstring[0]=0 : Empty -> first increment */
          irrstring[irrstring[0]] = i;
          if (!i) stringvalid = 1;
        }
        TCCR0 = _BV(CS01);
        modus = oldmodus;
      }
    }
}
void ir_send(u08 *irbuf_) {                                       /**< send an ir pulse */
  if ((modus == ir_s)|(modus == ir_r)) return;
  irsbuf = irbuf_;
  TCCR0 = _BV(CS00);                                              /* prescaler = 0 */
  TCNT0 = 255;
  irbit = 0;
  irtime = irbittime;
  TCCR2 = 0;
  oldmodus = modus;
  modus = ir_s;
}
void start_ir_r() {
  if ((modus == ir_s)|(modus == ir_r)) return;
  TCCR0 = _BV(CS00);
  TCNT0 = 255;
  for (irbit=1;irbit < IRMLENGTH;irbit++)
    irrbuf[irbit] = 0x00;
  irrbuf[0]=IRMLENGTH;
  irbit = 8;
  irtime = 0;
  TCCR2 = 0;
  oldmodus = modus;
  modus = ir_r;
}
void ir_string_send(u08 *se) {
  senddata=se;
  oldmodus2 = modus;
  modus = irstring_s;
  sendready = 0;
  WAITIR;
}
void do_ir_string() {
  static u16 timeir;
  u08 data;
  if (timeir > IRDATABYTETIME) {
    sendready = 1;
    modus = oldmodus2;
    timeir = IRDATABYTETIME-1;
    return;
  }
  if (timeir++ == IRDATABYTETIME) {
    data = senddata[0];
    ir_send(val_to_ir(data));
    if (data != 0) timeir = 0;
    senddata++;
  }
}
u08 ir_comp(irdatatype cmp_target, irdatatype read_ir) {
  u08 i = 127;
  u08 i2;
  for (i2=8;i2<=(cmp_target[0]<<3);i2++)
    if (cmp_target[i2 >> 3] & (1 << (i2 % 8)))
          if (read_ir[i2 >> 3] & (1 << (i2 % 8))) i++; else i--;
  return i;
}
#ifdef STATION
void ir_and(irdatatype res, irdatatype and2) {
  u08 i;
  for (i=1;i<=res[0];i++) {
    res[i] &= and2[i];
  }
}
#endif