//
// C Implementation: stationu
//
// Description:
//
//
// Author: Leonhard Klein <leoklein@gmx.net>, (C) 2004

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "basic.h"
#include "stationu.h"
#include "ir.h"


const u08 __ATTR_PROGMEM__ Pmenu0[]="\033[2J----------------------\n\r"
                                    "- Welcome to Leobot  -\n\r"
                                    "----------------------\n\r"
                                    "Keys:\n\r"
                                    " 1: Change loading current\n\r"
                                    " 2: Upload IR-Data\n\r"
                                    " 3: See & learn incoming ir-Data\n\r"
                                    " 4: Communicate to robot\n\r"
                                    " a,c,e: set mode of station\n\r\n\r"
                                    "Actual mode: \0";
const u08 __ATTR_PROGMEM__ Pmenu1[]="\033[2JSet loading current: (+ and - to change, 0 to leave)\n\r"
                                    " Actual Value: \0";
const u08 __ATTR_PROGMEM__ Pmenu2[]="\033[2JPress 0 to abort, any other key to continue\n\r\0";
const u08 __ATTR_PROGMEM__ Pmenu3[]="\033[2JIncoming IR-Data:\n\r"
                                    "Keys: 0: Leave, +-: Ir-speed, 1..4: Learn string\n\r"
                                    "      r: Reset learned strings, c: Test learned\n\r"
                                    "STRING UNTIL NOW -->\0";
const u08 __ATTR_PROGMEM__ Pmenu4[]="\033[2JROBOT-SPEAK: (send empty to leave)\n\r\0";
const u08 __ATTR_PROGMEM__ Pempty[]="\n\rLOADING STATION EMPTY!!\0";
const u08 __ATTR_PROGMEM__ Pstart[]="Starting up\0";
const u08 __ATTR_PROGMEM__ Pirpulse[]="\n\rIR PULSE-->\0";
const u08 __ATTR_PROGMEM__ Pirstring[]="\n\rIR STRING -->\0";

u08 sendbuf[IRMSLENGTH+1];
u08 recbuf[IRKEYS][IRMLENGTH];
u08 pulsecount;
u08 ircmp[IRKEYS];
u08 oldsw7;
u08 ircontbittime;
u08 robotneedspower;

#define WAIT10 for (i=0; i < 100; i++) for (i2 = 0; i2 < 100; i2++) { _NOP; }


SIGNAL(SIG_OVERFLOW0)                                    /* Timer0 Overflow for threating; normal op: 2 kKz */
{
  static u16 time;
  static u16 empty_c, diff_c;
  static s16 diff;
  s16 ocr1atemp;
  cli();						    /* SIGNAL doesn't stop all interrupts correctly (?) */
  if ((modus == ir_r)|(modus == ir_s)) {                    /* common check for common timing */
    TCNT0 = 255-112;                                        /* 13.888 = 36000Hz  111 */
    do_ir();
    sei();
    return;
  }
  time++;
  if (modus == empty) {
    if (time % _BV(12)==0) {
      uart_send_p(Pempty);
      RLEDON;
    }
    if (time % _BV(12) == 100) RLEDOFF;
    OCR1A = 0;
    return;
  }
  if (time % _BV(3) == 1) {
    start_ad(vcc);
    if (vcc_value < 600) {
      if (empty_c++ > 100) {
        lastkey = 1;
        modus = empty;
        empty_c = 0;
      }
    } else
      empty_c =0;
  }
  if (time % _BV(3) == 4)
    start_ad(vout);
  if (time % _BV(7) == 6) {
    diff = (vcc_value - vout_value + diff) >> 1;              /* mean of the last two values */
    if ((diff) < loadcur)
      ocr1atemp = OCR1A + 4;
    else
      ocr1atemp = OCR1A - 4;
    if (ocr1atemp > 255)
      ocr1atemp = 255;
    if (ocr1atemp < 0)
      ocr1atemp = 0;
    OCR1A = ocr1atemp;
  }

  RLEDOFF;
  GLEDOFF;
  if (modus == irstring_s) {
    do_ir_string();
    RLEDON;
  } else {
    if (SW7 != oldsw7) modus = (modus==connected)?away:connected;
    if (modus == away) {
      DO_PING;                        /* 1kHz */
      if (diff > 30) {
        lastkey = 1;
        modus = connected;
        STOP_PING;
      }
    } else if (modus == connected) {
      GLEDON;
      if ((diff < 30) & (robotneedspower)) diff_c++; else diff_c = 0;
      if (diff_c > 8000) modus = away;
      if ((!pulsevalid) & (ISIRON)) {
        if (pulsecount > 5) start_ir_r();
        pulsecount = 0;
      } else if (pulsecount < 100) pulsecount++;
    }
  }
  oldsw7 = SW7;
  sei();
}

int main(void) {
  u08 menumode=0;
  u08 recmode;
  u08 oldmenumode;
  u16 i,i2;
  s08 s08_1;
  u08 irspos;

  modus = connected;
  admodus = none;
  vcc_value = 151;
  vout_value = 151;
  irerror = 0;
  oldsw7 = SW7;
  loadcur = 71;                                       /* at 2,2Ohm  350mA */
  irbittime = 12;
  initchip();
  CLRSCR;
  uart_send_p(Pstart);
#ifndef TEST
  for (i=0; i < 8;i++) {
    for (i2=0; i2 < 50000;i2++) _NOP;
    uart_send(".");
    if (i%2==1) {
      GLEDON;
      RLEDOFF;
    } else {
      RLEDON;
      GLEDOFF;
    }
  }
  GLEDOFF;
  RLEDOFF;
#endif
  while (1) {                                         /* Terminal */
    switch (menumode) {
    case 0:
      uart_send_p(Pmenu0);
      if (modus==away)
        uart_send("away\n\r");
      else if (modus==connected)
        uart_send("connected\n\r");
      else if (modus == empty)
        uart_send("empty\n\r");
      else uart_send("IR-Modus\n\r");
      break;
    case 1:
      uart_send_p(Pmenu1);
      uart_send_int(loadcur, 10);
      uart_send("\n\rvcc ->");
      uart_send_int(vcc_value, 10);
      uart_send("\n\rvout->");
      uart_send_int(vout_value, 10);
      uart_send("\n\rOCR ->");
      uart_send_int(OCR1A, 10);
      break;
    case 2:
      uart_send_p(Pmenu2);
      break;
    case 3:
      if (lastkey) {
        CLRSCR;
        uart_send_p(Pmenu3);
        irrstring[irrstring[0]+1]=0;
        uart_send_pascal(irrstring);
        uart_send("\n\rIR-Bittime: ");
        uart_send_int(irbittime,10);
        for (i2=0;i2 < IRKEYS;i2++) {
          uart_send("\n\rRECBUF");
          uart_send_int(i2+1,10);
          uart_send("-->");
          for (i=0; (i<=recbuf[i2][0]); i++) {
            uart_send_int(recbuf[i2][i], 16);
          }
        }
      }
      break;
    case 4:
      if (oldmenumode != menumode) {
        CLRSCR;
        uart_send_p(Pmenu4);
        uart_send_pascal(sendbuf);
      }
      uart_send("\033[2K\rStation->");
      uart_send_pascal(sendbuf);
      break;
    }
    pulsevalid = 0;
    stringvalid = 0;
    lastkey=0;
    oldmenumode = menumode;
    sei();
    do {
#ifdef TEST
      SIG_OVERFLOW0();
      if (modus > empty) SIG_ADC();
#endif
      _NOP;
    } while (!((lastkey)|(pulsevalid)|(stringvalid)));
    cli();
    if (stringvalid) {
      for (i = 0; i < irrstring[0]; i++) {
        if (irrstring[i]=='E') robotneedspower = 1;
        if (irrstring[i]=='F') robotneedspower = 0;
      }
    }
    switch (menumode) {
    case 0:
      switch (lastkey) {
      case '1':
      case '2':
      case '3':
      case '4':
        menumode = lastkey-'0';
        break;
      case 'a':
        modus = away;
        break;
      case 'c':
        modus = connected;
        STOP_PING;
        break;
      case 'e':
        modus = empty;
        break;
      }
      break;
    case 1:
      switch (lastkey) {
      case '+':
        loadcur++;
        break;
      case '-':
        loadcur--;
        break;
      case '0':
        menumode = 0;
        break;
      }
      break;
    case 2:
      if ((lastkey != '0') & (lastkey != 0)) {
        irbittime = IRDATABITTIME;                 // standard timing for data transmission
        uart_send("Transmitting..");
        sendbuf[0]='I';
        sendbuf[1]=0;
        ir_string_send(sendbuf);
        WAIT10;
        for (irspos = 0; irspos < IRKEYS*IRMLENGTH; irspos++) {
          itoa16((recbuf[irspos / IRMLENGTH][irspos % IRMLENGTH]), sendbuf);
          sendbuf[2]=0;
          if (irspos%20==0) uart_send("\n\r");
          uart_send(".");
          ir_string_send(sendbuf);
          WAIT10;
        }
        itoa16(ircontbittime, sendbuf);
        ir_string_send(sendbuf);
        WAIT10;
        sendbuf[1] = 'Q';
        ir_string_send(sendbuf + 1);
        sendbuf[0] = 0;
        uart_send("\n\rDone");
        lastkey = 0;
        sei();
        while (!lastkey);
        if (stringvalid) uart_send(irrstring+1);
        cli();
      }
      menumode = 0;
      break;
    case 3:
      if (pulsevalid) {
        uart_send_p(Pirpulse);
        for (i=0; (i <= irrbuf[0]); i++) {
          uart_send_int(irrbuf[i], 16);
        }
        if (recmode == 0) {
          uart_send("\n\rCMP:");
          i2 = 0;
          for (i=0;i < IRKEYS;i++) {
            uart_send_int(ircmp[i]=ir_comp(recbuf[i],irrbuf),10);
            if (i2<ircmp[i]) i2 = ircmp[i];
          }
          uart_send(" WINNER:----->");
          for (i = 0; i < IRKEYS; i++)
            if (i2 == ircmp[i]) uart_send_int(i,10);
        } else if (recmode < IRKEYS+1) {
          if (recbuf[recmode-1][0] == IRMLENGTH) {
            for (i2 = 0; i2 < IRKEYS; i2++) recbuf[i2][0] = irrbuf[0];
          }
          s08_1 = (recbuf[recmode-1][0]-irrbuf[0]);
          if ((s08_1 > -2) && (s08_1 < 2))
            ir_and(recbuf[recmode-1], irrbuf);
        }
        irrbuf[0]=0;
      }
      if (stringvalid) {
        uart_send_p(Pirstring);
        uart_send_pascal(irrstring);
        irrstring[0] = 0;
      }
      switch (lastkey) {
      case '+': if (irbittime <100) irbittime++;
        break;
      case '-': if (irbittime > 2) irbittime--;
        break;
      case '0': menumode = 0;
        break;
      case '1':
      case '2':
      case '3':
      case '4':
        recmode = lastkey - '0';
        break;
      case '9': recmode = 9;
        break;
      case 'c': recmode = 0;
        break;
      case 'r': for (i2 = 0; i2 < IRKEYS; i2++) {
          for (i = 1; i < IRMLENGTH; recbuf[i2][i++] = 0xff);
          recbuf[i2][0] = IRMLENGTH;
        }
	ircontbittime = irbittime;
        break;
      }
      break;
    case 4:
      irbittime = IRDATABITTIME;                 // standard timing for data transmission
      if (stringvalid) {
        uart_send("\033[2K\rRobot->");
        uart_send_pascal(irrstring);
	if (irrstring[1]=='S') {
	  menumode = 0;
	  modus = away;
	}
        irrstring[0]=0;
        uart_send("\n\r");
      }
      switch (lastkey) {
      case 13:
        if (!sendbuf[0])
          menumode = 0;
        else {
          sendbuf[sendbuf[0]+1]=0;
          uart_send("\n\rSending...");
          ir_string_send(sendbuf+1);
          sendbuf[0] = 0;
        }
      case 0:
        break;
      case 8:
      case 127:
        if (sendbuf[0]>0) sendbuf[0]--;
        break;
      default:
        if (sendbuf[0]<IRMSLENGTH) sendbuf[0]++;
        sendbuf[sendbuf[0]]=lastkey;
      }
    }
  }
}
