//
// C main: stationdoc
//
// Description: Description of the station
//
//
// Author: Leonhard Klein <leoklein@gmx.net>, (C) 2004

/*! \mainpage THE LEO ROBOT PROJECT
    This is the Code of the loading- and dockingstation of the robot.

    Thoughts about the ir-data-transmittion:
    do_ir() writes to irrbuf which is a global variable of irdatatype [IRMLENGTH]! irbuf can be modified by do_* for checking of new data (irrbuf[0] != 0 -> Data received).
    If the received datas match the robot data format they will be written to irrstring, where the first char is the length. This length is incremented every received char, so it's up to you to reset the string.

    Each impulse must be startet with start_r_ir();


    For sending the irsbuf[] is used, which is a pointer to a string given by ir_send. (no own Memory). This is initialized with ir_send(irdatatype *irtosend). These datas may not be valid robot data format.
    For sending a string of datas use ir_send_string(u08 *stringtosend). This changes the modus to irstring_s and calles ir_send(val_to_ir(..)) for each char until a \\0 is found (the \\0 is transmitted also)



    The fundamental routines read and write to some special strings(recdata, senddata).
    While receiving the ir_to_val is executed and if data is found it is appended to recstring.
    Sending is done via ir_string_send(u08 *se) which reads the *se while sending.


 */

/* FILES */
/*! \file stationdoc.c
    \brief This file contains the documentation for the project
*/

/*! \file basic.h
    \brief Contains fundamental typedefinitions
*/

/*! \file station.c
    \brief The software of the loading station.

    The loading station has a functionality to transmit data from and to the robot. Furthermore it has a Terminal adapter over the serial port by which the datas are transmitted to the PC and it is adjusted.

    Last but not least it regulates the current for the accu of the robot and checks the own voltage.
*/

/*! \file stationu.h
    \brief Helper functions for station.
*/

/*! \file ir.c
    \brief The functions for the ir-transmitting and receiving. Hopefully the same file as the Robot gets.
*/


/* FUNCTIONS */

/*! \fn SIGNAL(SIG_OVERFLOW0)
    \brief This function is called by the Timer0 which is the general "tasking" clock of the station

    There are two modes of the Timer used: the "normal" timing where the prescaler divides the clock by 8 and the "fast" without prescaler. The fast mode is used for the transmitting and receiving IR-routines. While these are active the normal functions are disabled. (IR needs a clock of 36kHz).
*/

/*! \fn SIGNAL(SIG_ADC)
    \brief Here the results of the ADC are evaluated.

    This function is called when the ADC is ready. Which AD-Channel was used and for what use is determined by admodus
*/

/*! \fn main()
    \brief The main program. Does some initialisation and keeps running the terminal.
*/


/*! \fn extern void start_ir_r();
    \brief starts the recieving of an IR Pulse.

    This will last maximum (irlength * irbittime / 36000) while nothing else will be computated.
    The result will be writte to irbuf
*/

/*! \fn extern void ir_send(u08 *irbuf_)
    \brief starts the sending of an IR Pulse.

    This will last (irbuf_[0] * irbittime /36000) while nothing else will be computated. The data is given in irbuf_
*/
/*! \fn u08 *val_to_ir(u08 dat)
   \brief Converts a byte-value to an ir-impuls stored in irdata
*/
/*! \fn u08 ir_to_val(irdatatype data)
   \brief Converts and returns an ir-impuls as value
*/
/*! \fn void do_ir()
   \brief The main Ir-Routine. Does sending, receiving and possible converting to a string.

   The results of receiving are stored in irbuf and irstring. The values pulsevalid and stringvalid return if any data was received.
*/
/*! \fn u08 ir_comp(irdatatype cmp_target, irdatatype read_ir)
   \brief Compares two IR pulses.

   This is used for ircont. The bigger the result the narrower the pulses.
   \param read_ir
   The received pulse.
   \param cmp_target
   The stored "and"-array. It is generated by the loading station.
*/
/*! \fn void ir_and(irdatatype res, irdatatype and2)
   \brief builds a mask for the ir-pulses. Only important in the Station.
*/
/*! \var irerror
    \brief returns the error-code for the ir_to_data() function. 0 means ok, and that the datas were a robotcode.
*/
/*! \var irbittime
    \brief the cycles per bit for the ir functions. The basic frequency is 36kHz.
*/
/*! \var stringvalid
   \brief Is set 1 if a valid string is received. Set it to 0 if computated. But also reset irrstring[0]!
*/
/*! \var sendready
   \brief This variable is used to wait for the IR-String send routines. It's used in WAITIR.
*/


/*! \type irdatatype
    \brief a container type for the impulse.
    The format of the ir-impulses is: [length],[1],[2],[3],...[<length-1>]. Where length is between 1 and IRMLENGTH.
    The bits are in msb order and from left to right. f.i. "04 0a 0a a0" is legal string where every bit is sent.
*/
/*! \def ISIRON
    \brief a Makro for checking the ir-module. 0 means ir is off 1 means on.
*/
