//
// C Implementation: stationu
//
// Description:
//
//
// Author: Leonhard Klein <leoklein@gmx.net>, (C) 2004

#include "stationu.h"

volatile u08 lastkey;

SIGNAL(SIG_UART_RECV)      /* signal handler for receive complete interrupt */
{  lastkey = UDR;
}

void uart_send(u08 *buf) {
  u08 i=0;
  while ((buf[i])|(i>250)) {
#ifndef TEST
    while (!(UCSRA & _BV(UDRE))) ;
#endif
    UDR = buf[i++];
  }
}
void uart_send_pascal(u08 *buf) {
  buf[buf[0]+1]=0;
  uart_send(buf+1);
}
void uart_send_char(u08 val) {
#ifndef TEST
  while (!(UCSRA & _BV(UDRE))) ;
#endif
  UDR = val;
}

void uart_send_p(u08 buf[]) {
  u08 i=0;
  u08 ch; 
  while ((ch = pgm_read_byte_near(buf+(i++)))|(i>250)) {
#ifndef TEST    
    while (!(UCSRA & _BV(UDRE))) ;
#endif
    UDR = ch;
  }
}

void uart_send_int(s16 int1, u08 radix) {
  u08 *intstr = "\0\0\0\0\0\0\0\0";
  itoa(int1, intstr, radix);
  if ((radix==16)&(int1<16))
    uart_send("0");
  uart_send(intstr);
  uart_send(" ");
}

void initchip() {
  DDRB = _BV(DDB0) | _BV(DDB1) | _BV(DDB4);
  PORTC = _BV(PC4) | _BV(PC3);                        /* Pull-Up's for dip-switch */
  DDRD = _BV(DD7);
  UCSRB = _BV(RXCIE) | _BV(RXEN) | _BV(TXEN);          /* enable RxD/TxD and ints */
  UBRRL = (u08)UART_BAUD_SELECT;                       /* set baud rate */

  TCCR1A= _BV(COM1A1)|_BV(COM1B1)|_BV(WGM10);          /* PWM 8bit*/
  TCCR1B= _BV(CS01);                                   /*  /1 => 16kHz */
  TCNT1 = 0;
  OCR1A = 255;

  TIMSK = _BV(TOIE0);
  TCNT0 = 0;
  TCCR0 = _BV(CS01);                                  /* Clock / 8 intervall is 4m/256*8~=2k */
}

void start_ad(enum admodtype adm) {
  if (admodus == none) {
    admodus = adm;
    if (adm == vcc)
      ADMUX = _BV(REFS0) + 2;                     /* voltage of accu */
    if (adm == vout)
      ADMUX = _BV(REFS0) + 1;                     /* Output voltage for current measure */
    ADCSRA = _BV(ADEN)|_BV(ADSC)|_BV(ADIE) + 3;   /* enable single mode */
  } else
    uart_send("AD-ERROR\n\r\0");
}

SIGNAL(SIG_ADC) {
  if (admodus == vcc)
    vcc_value = ADC;
  if (admodus == vout)
    vout_value = ADC;
  ADCSRA = 0;                                         /* disable AD-Converter */
  admodus = none;
}
void itoa16(u08 val, u08 *buf) {
  u08 i1, i2;
  i1 = (val >> 4);
  i2 = (val & 0x0f);
  if (i1 > 9) i1 += 7;
  if (i2 > 9) i2 += 7;
  buf[0] = i1+'0';
  buf[1] = i2+'0';
}